@@ -8,6 +8,8 @@ import logging
 import plotly
 from pathlib import Path
 from itertools import chain
+from plotly import subplots
+from plotly.subplots import make_subplots
 
 
 class Region(object):
@@ -100,7 +102,9 @@ def get_args():
     parser.add_argument("-g", "--gtf", help="add annotation based on a gtf file")
     parser.add_argument("-b", "--bed", help="add annotation based on a bed file")
     parser.add_argument(
-        "-f", "--fasta", help="required when --window is an entire chromosome, contig or transcript"
+        "-f",
+        "--fasta",
+        help="required when --window is an entire chromosome, contig or transcript",
     )
     parser.add_argument(
         "--simplify",
@@ -108,7 +112,9 @@ def get_args():
         action="store_true",
     )
     parser.add_argument(
-        "--split", help="split, rather than overlay the methylation tracks", action="store_true"
+        "--split",
+        help="split, rather than overlay the methylation tracks",
+        action="store_true",
     )
     parser.add_argument("--static", help="Make a static image of the browser window")
     parser.add_argument(
@@ -117,10 +123,16 @@ def get_args():
         action="store_true",
     )
     parser.add_argument(
-        "--smooth", help="Rolling window size for averaging frequency values", type=int, default=5
+        "--smooth",
+        help="Rolling window size for averaging frequency values",
+        type=int,
+        default=5,
     )
     parser.add_argument(
-        "--dotsize", help="Control the size of dots in the per read plots", type=int, default=4
+        "--dotsize",
+        help="Control the size of dots in the per read plots",
+        type=int,
+        default=4,
     )
     parser.add_argument(
         "--minqual",
@@ -236,6 +248,10 @@ def file_sniffer(filename):
         return "nanopolish_freq"
     if len(header.split("\t")) == 4:
         return "bedgraph"  # there is no header, but the file is tab separated and has 4 fields
+    if len(header.split("\t")) == 14:  # file generated by `modbam2bed --extended`
+        return "bedmethyl_extended"  # there is no header, but the file is tab separated and has 14 fields
+    if len(header.split("\t")) == 18:  # file generated by modkit --only-tabs
+        return "bedmethyl"  # there is no header, but the file is tab separated and has 18 fields
     sys.exit(f"\n\n\nInput file {filename} not recognized!\n")
 
 
